#!/bin/bash
#-------------------------------------------------------------------------------
#   Copyright 2008-2012 Hewlett Packard Co., All Rights Reserved.
#-------------------------------------------------------------------------------
#
#   Script to backup and restore the various configurations of Network Optimizer
#   
#   written by Abhirup Kundu
#
#   July 30, 2015
#-------------------------------------------------------------------------------

# trap the ^C and ^D during the backup/restore
trap '' SIGINT
trap '' SIGQUIT

# usage display
usage()
{
    echo "Usage: $0 [-b | --backup ] [-r | --restore ] [IP] [OPTION]" 1>&2
    echo "Take backup, or restore the Network Optimizer Configurations"
    echo
    echo "    -b, --backup    take the backup of Network Optimizer data"
    echo "    -r, --restore   restore the Network Optimizer data"
    echo "OPTION"
    echo "    -d, --dir       absolute path of the directory where the backup file will be saved"
    echo "    -n, --name      backup file name"
    echo "    -f, --file-path absolute path of the backed up file to restore"
    echo "    -h, --help      display this help and exit"
    exit 1
}

OPTION="$1"
IP="$2"
TOKEN=""
BASE_URL="https://$IP:8443/sdn/ui/lyncqos/app/rs"
OPT_TEMP="/opt/temp"

function readSDNCreds() {
    for ((i=1;i<=3;++i)) ; do
        echo -n "Enter SDN Controller Username: "
        read SDN_UNAME
        echo -n "Enter Password: "
        read -s SDN_PASS
        echo ""
        AUTH_JSON="{ \"login\": { \"user\": \"$SDN_UNAME\",\"password\": \"$SDN_PASS\" } }"
        AUTH_URL="https://$IP:8443/sdn/v2.0/auth"
        ACTION_OUT=$(curl -s -k --noproxy $IP -X POST $AUTH_URL -H "Content-Type: application/json" --data-binary "$AUTH_JSON")
        TOKEN=$(echo $ACTION_OUT | sed "s/^\(.*\?\(\"token\":\"\?\([^,}\"]*\)\)\)\?.*$/\3/")
        unset SDN_UNAME; unset SDN_PASS;
        if [ -n "$TOKEN" ] ; then
            return 0;
        elif [ -z "$TOKEN" ] ; then
            if [ $i == 3 ] ; then
                echo "ERROR: Invalid SDN Credentials, aborting Config Snapshot/Restore" && exit 1;
            fi
            echo "ERROR: Invalid SDN Credentials, Re-Enter"
        fi
    done
}

#-------------------------------------------------------------------------------
# APIs for each configuration calling GET and POST / PUT
#-------------------------------------------------------------------------------

function hardphoneVlans() {
    if [ "$1" == GET ]; then
        HPVLANS_GET_URL="$BASE_URL/hardphoneVlans"
        HPVLANS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $HPVLANS_GET_URL) > /dev/null 2>&1
        echo $HPVLANS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
hpvlans=obj["vlans"];
postJson="{\"lyncHardphone\":[{\"vlans\":\"" + hpvlans + "\"}]}";
print postJson;
        ' > $OPT_TEMP/hardphoneVlans

    elif [ "$1" == POST ]; then
        HPVLANS_PUT_URL="$BASE_URL/hardphoneVlans"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/hardphoneVlans --url $HPVLANS_PUT_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function edgeClassifyingVlans() {
    if [ "$1" == GET ]; then
        EDGEVLANS_GET_URL="$BASE_URL/edgeClassifyingVlans"
        EDGEVLANS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $EDGEVLANS_GET_URL) > /dev/null 2>&1
        echo $EDGEVLANS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
ecvlans=obj["vlans"];
postJson="{\"lyncEdgeClassifying\":[{\"vlans\":\"" + ecvlans + "\"}]}";
print postJson;
        ' > $OPT_TEMP/edgeClassifyingVlans

    elif [ "$1" == POST ]; then
        EDGEVLANS_PUT_URL="$BASE_URL/edgeClassifyingVlans"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/edgeClassifyingVlans --url $EDGEVLANS_PUT_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function gateways() {
    if [ "$1" == GET ]; then
        GATEWAYS_GET_URL="$BASE_URL/gateways"
        GATEWAYS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $GATEWAYS_GET_URL) > /dev/null 2>&1
        echo $GATEWAYS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
postJson="{\"gatewayadd\":["
gateways=obj["lyncgateways"];
for gateway in gateways:
    dpid=gateway["gatewayDpIdKey"];
    port=gateway["gatewayPortKey"];
    ip=gateway["gatewayIpKey"];
    subnet=gateway["gatewaySubnetKey"];
    postJson=postJson + "\"" + str(dpid) + "/" + str(port) + "/" + str(ip) + "/" + str(subnet) + "\",";
postJson=postJson.rstrip(",");
postJson = postJson + "],\"gatewaydel\":[]}";
print postJson;
        ' > $OPT_TEMP/gateways

    elif [ "$1" == POST ]; then
        GATEWAYS_PUT_URL="$BASE_URL/gateways"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/gateways --url $GATEWAYS_PUT_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function anonymous() {
    if [ "$1" == GET ]; then
        ANONYMOUS_GET_URL="$BASE_URL/anonymous"
        ANONYMOUS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $ANONYMOUS_GET_URL) > /dev/null 2>&1
        echo $ANONYMOUS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
anonymous=obj["lyncAnonymityMode"];
postJson="{\"anonymity\":[{\"enable\":\"" + str(anonymous) + "\"}]}";
print postJson;
        ' > $OPT_TEMP/anonymous

    elif [ "$1" == POST ]; then
        ANONYMOUS_PUT_URL="$BASE_URL/anonymous"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/anonymous --url $ANONYMOUS_PUT_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function lyncSdnManagers() {
    if [ "$1" == GET ]; then
        SDNMGRS_GET_URL="$BASE_URL/sdnManagers"
        SDNMGRS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $SDNMGRS_GET_URL) > /dev/null 2>&1
        echo $SDNMGRS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
postJson="{\"lyncserveradd\":["
sdnmgrs=obj["lyncsdnmanagers"];
for sdnmgr in sdnmgrs:
    ip=sdnmgr["lyncSvrIpAddrKey"];
    postJson=postJson + "\"" + str(ip) + "\",";
postJson=postJson.rstrip(",");
postJson = postJson + "],\"lyncserverdel\":[]}";
print postJson;
        ' > $OPT_TEMP/sdnManagers

    elif [ "$1" == POST ]; then
        SDNMGRS_PUT_URL="$BASE_URL/sdnManagers"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/sdnManagers --url $SDNMGRS_PUT_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function lyncFrontEndServers() {
    if [ "$1" == GET ]; then
        FESERVERS_GET_URL="$BASE_URL/frontEndServer"
        FESERVERS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $FESERVERS_GET_URL) > /dev/null 2>&1
        echo $FESERVERS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
postJson="{\"lyncserveradd\":["
feservers=obj["lyncservers"];
for feserver in feservers:
    ip=feserver["lyncSvrIpAddrKey"];
    postJson=postJson + "\"" + str(ip) + "\",";
postJson=postJson.rstrip(",");
postJson = postJson + "],\"lyncserverdel\":[]}";
print postJson;
        ' > $OPT_TEMP/frontEndServer

    elif [ "$1" == POST ]; then
        FESERVERS_PUT_URL="$BASE_URL/frontEndServer"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/frontEndServer --url $FESERVERS_PUT_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function globalQos() {
    if [ "$1" == GET ]; then
        GLOBALQOS_GET_URL="$BASE_URL/dscp"
        GLOBALQOS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $GLOBALQOS_GET_URL) > /dev/null 2>&1
        echo $GLOBALQOS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
postJson="{\"dscpconfig\":[{"
dscp=obj["dscpconfig"];
ctrl=dscp["dscpcontrol"];
voice=dscp["dscpvoice"];
video=dscp["dscpvideo"];
app=dscp["dscpappshare"];
be=dscp["dscpbe"];
ctrlpri=dscp["controlpri"];
voicepri=dscp["voicepri"];
videopri=dscp["videopri"];
apppri=dscp["apppri"];
bepri=dscp["bepri"];
postJson=postJson + "\"dscpcontrol\":" + str(ctrl) + ",\"dscpvoice\":" + str(voice) + ",\"dscpvideo\":" + str(video) + ",\"dscpappshare\":" + str(app) + ",\"dscpbe\":" + str(be) + ",";
postJson=postJson + "\"controlpri\":" + str(ctrlpri) + ",\"voicepri\":" + str(voicepri) + ",\"videopri\":" + str(videopri) + ",\"apppri\":" + str(apppri) + ",\"bepri\":" + str(bepri);
postJson=postJson + "}]}";
print postJson;
        ' > $OPT_TEMP/globalQos

    elif [ "$1" == POST ]; then
        GLOBALQOS_POST_URL="$BASE_URL/dscp"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/globalQos --url $GLOBALQOS_POST_URL --header "X-Auth-Token: $TOKEN" -k #> /dev/null 2>&1
    fi
}

function locationQos() {
    if [ "$1" == GET ]; then
        LOCQOS_GET_URL="$BASE_URL/subnetlist"
        LOCQOS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $LOCQOS_GET_URL) > /dev/null 2>&1
        echo $LOCQOS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
postJson=json.dumps(obj["subnetlist"]);
print postJson;
        ' > $OPT_TEMP/locationQos

    elif [ "$1" == POST ]; then
        LOCQOS_POST_URL="$BASE_URL/subnetlist"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/locationQos --url $LOCQOS_POST_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function qoeAlarm() {
    if [ "$1" == GET ]; then
        ALARM_GET_URL="$BASE_URL/qoeAlarmConfig"
        ALARM_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $ALARM_GET_URL) > /dev/null 2>&1
        echo $ALARM_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
alarm=json.dumps(obj);
postJson="{\"qoeAlarmConfig\":[" + alarm + "]}";
print postJson;
        ' > $OPT_TEMP/qoeAlarmConfig

    elif [ "$1" == POST ]; then
        ALARM_POST_URL="$BASE_URL/qoeAlarmConfig"
        curl --noproxy $IP -X POST -d@$OPT_TEMP/qoeAlarmConfig --url $ALARM_POST_URL --header "X-Auth-Token: $TOKEN" -k > /dev/null 2>&1
    fi
}

function bandwidthTemplate() {
    if [ "$1" == GET ]; then
        BANDTMPL_GET_URL="$BASE_URL/bandwidthtemplates"
        BANDTMPL_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $BANDTMPL_GET_URL) > /dev/null 2>&1
        echo $BANDTMPL_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
templates=obj["bandwidthTemplates"];
for template in templates:
    print "{\"bandwidthtemplate\":" + json.dumps(template) + "}";
        ' > $OPT_TEMP/bandwidthtemplates

    elif [ "$1" == PUT ]; then
        BANDTMPL_PUT_URL="$BASE_URL/bandwidthtemplates"
        while read TEMPLATE; do
            [ -z "$TEMPLATE" ] && continue
            echo $TEMPLATE > $OPT_TEMP/bandwidthtemplates-put
            curl --noproxy $IP -X PUT -d@$OPT_TEMP/bandwidthtemplates-put --url $BANDTMPL_PUT_URL -H "X-Auth-Token: $TOKEN" -H "Content-Type: application/json" -k > /dev/null 2>&1
            rm -f $OPT_TEMP/bandwidthtemplates-put
        done < $OPT_TEMP/bandwidthtemplates
    fi
}

function granularQos() {
    if [ "$1" == GET ]; then
        GRANQOS_GET_URL="$BASE_URL/staticqos"
        GRANQOS_CURL_OUT=$(curl -s -k --noproxy $IP -X GET --header "X-Auth-Token:$TOKEN" $GRANQOS_GET_URL) > /dev/null 2>&1
        echo $GRANQOS_CURL_OUT | python -c '
import json,sys;
obj=json.load(sys.stdin);
policies=obj["policies"];
for policy in policies:
    print json.dumps(policy);
        ' > $OPT_TEMP/granularQos

    elif [ "$1" == POST ]; then
        GRANQOS_POST_URL="$BASE_URL/staticqos"
        while read POLICY; do
            [ -z "$POLICY" ] && continue
            echo $POLICY > $OPT_TEMP/granularQos-post
            curl --noproxy $IP -X POST -d@$OPT_TEMP/granularQos-post --url $GRANQOS_POST_URL -H "X-Auth-Token: $TOKEN" -H "Content-Type: application/json" -k > /dev/null 2>&1
            rm -f $OPT_TEMP/granularQos-post
        done < $OPT_TEMP/granularQos
    fi
}

function initiate_backup() {
    readSDNCreds

    [ -e $OPT_TEMP ] && rm -rf $OPT_TEMP
    mkdir -p $OPT_TEMP

    hardphoneVlans GET
    edgeClassifyingVlans GET
    gateways GET
    anonymous GET
    lyncSdnManagers GET
    lyncFrontEndServers GET
    globalQos GET
    bandwidthTemplate GET
    locationQos GET
    qoeAlarm GET
    granularQos GET

    cd $OPT_TEMP
    zip -q $ZIP_NAME.zip *
    mkdir -p $TARGET_DIR
    mv $ZIP_NAME.zip $TARGET_DIR
    cd -
    rm -rf $OPT_TEMP
}

function initiate_restore() {
    readSDNCreds

    [ -e $OPT_TEMP ] && rm -rf $OPT_TEMP
    mkdir -p $OPT_TEMP

    unzip -q $ZIP_FILE -d $OPT_TEMP

    hardphoneVlans POST
    edgeClassifyingVlans POST
    gateways POST
    anonymous POST
    lyncSdnManagers POST
    lyncFrontEndServers POST
    globalQos POST
    bandwidthTemplate PUT
    locationQos POST
    qoeAlarm POST
    granularQos POST

    rm -rf $OPT_TEMP
}

NUM_OF_ARGS=$#

if [ $NUM_OF_ARGS -eq 0 ]; then
    usage && exit 1;
elif [ $NUM_OF_ARGS -gt 6 ]; then
    echo "Error: Invalid arguments specified" && usage && exit 1;
fi

OPTION=$1
IP=$2

case "$OPTION" in
    --backup | -b )
        if [ $NUM_OF_ARGS -eq 2 ]; then
            
            # ./netopt-snapshot -b|--backup
            ZIP_NAME="netopt-snapshot"
            TARGET_DIR="/opt/"

            echo "Initiating Network Optimizer Configuration Snapshot"
            initiate_backup

            echo "Config Snapshot $ZIP_NAME.zip is present at $TARGET_DIR"
            exit 0;

        elif [ $NUM_OF_ARGS -eq 6 ]; then
            
            # ./netopt-snapshot -b|--backup -d|--dir <dir-name> -n|--name <zipname>
            DIR_ARG=$3
            TARGET_DIR=$4
            FNAME_ARG=$5
            ZIP_NAME=$6

            [ "$DIR_ARG" != -d ] && [  "$DIR_ARG" != --dir ] && \
                echo "Error: Invalid arguments specified" && usage && exit 1;

            [ "$FNAME_ARG" != -n ] && [ "$FNAME_ARG" != --name ] && \
                echo "Error: Invalid arguments specified" && usage && exit 1;

            TARGET_BASE=$(dirname $TARGET_DIR)
            [ ! -e $TARGET_BASE ] && echo "Error: Basepath $TARGET_BASE does not exist" && exit 1;

            echo "Initiating Network Optimizer Configuration Snapshot"
            initiate_backup

            echo "Config Snapshot $ZIP_NAME.zip is present at $TARGET_DIR"
            exit 0;

        fi
        echo "Error: Invalid arguments specified" && usage && exit 1;
    ;;

    --restore | -r )
        if [ $NUM_OF_ARGS -eq 2 ]; then

            # ./netopt-snapshot -r|--restore
            ZIP_FILE="/opt/netopt-snapshot.zip"
            [ ! -e $ZIP_FILE ] && echo "Error: Backed up file $ZIP_FILE does not exist" && exit 1;

            echo "Initiating Network Optimizer Configuration Restore"
            initiate_restore

            echo "Network Optimizer Configuration Restore completed"
            exit 0;

        elif [ $NUM_OF_ARGS -eq 4 ]; then
            FILE_PATH_ARG=$3
            ZIP_FILE=$4

            [ "$FILE_PATH_ARG" != "-f" ] && [ "$FILE_PATH_ARG" != --file ] && \
                echo "Error: Invalid arguments specified" && usage && exit 1;

            [ ! -e $ZIP_FILE ] && echo "Error: Backed up file $ZIP_FILE does not exist" && exit 1

            echo "Initiating Network Optimizer Configuration Restore"
            initiate_restore

            echo "Network Optimizer Configuration Restore completed"
            exit 0;
        fi
        echo "Error: Invalid arguments specified" && usage && exit 1;
    ;;

    --help | -h )
        echo "Help Menu - Options" && usage && exit 1;
    ;;

    -* )
        echo "Error: Invalid arguments specified" && usage && exit 1;
        exit 1
    ;;
esac
